# Copyright (c) 2015 Cloudbase Solutions SRL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock
from oslo_concurrency import processutils
from oslo_utils import importutils
from oslo_utils import strutils

from manila import exception
from manila.share.drivers.windows import winrm_helper
from manila import test


@ddt.ddt
class WinRMHelperTestCase(test.TestCase):
    _FAKE_SERVER = {'ip': mock.sentinel.ip}

    @mock.patch.object(importutils, 'import_module')
    def setUp(self, mock_import_module):
        self._winrm = winrm_helper.WinRMHelper()
        super(WinRMHelperTestCase, self).setUp()

    @ddt.data({'import_exc': None},
              {'import_exc': ImportError})
    @mock.patch.object(importutils, 'import_module')
    @ddt.unpack
    def test_setup_winrm(self, mock_import_module, import_exc):
        winrm_helper.winrm = None
        mock_import_module.side_effect = import_exc

        if import_exc:
            self.assertRaises(exception.ShareBackendException,
                              winrm_helper.setup_winrm)
        else:
            winrm_helper.setup_winrm()
            self.assertEqual(mock_import_module.return_value,
                             winrm_helper.winrm)
        mock_import_module.assert_called_once_with('winrm')

    @mock.patch.object(winrm_helper.WinRMHelper, '_get_auth')
    @mock.patch.object(winrm_helper, 'WinRMConnection')
    def test_get_conn(self, mock_conn_cls, mock_get_auth):
        mock_auth = {'mock_auth_key': mock.sentinel.auth_opt}
        mock_get_auth.return_value = mock_auth

        conn = self._winrm._get_conn(self._FAKE_SERVER)

        mock_get_auth.assert_called_once_with(self._FAKE_SERVER)
        mock_conn_cls.assert_called_once_with(
            ip=self._FAKE_SERVER['ip'],
            conn_timeout=self._winrm._config.winrm_conn_timeout,
            operation_timeout=self._winrm._config.winrm_operation_timeout,
            **mock_auth)
        self.assertEqual(mock_conn_cls.return_value, conn)

    @ddt.data({},
              {'exit_code': 1},
              {'exit_code': 1, 'check_exit_code': False})
    @mock.patch.object(strutils, 'mask_password')
    @mock.patch.object(winrm_helper.WinRMHelper, '_parse_command')
    @mock.patch.object(winrm_helper.WinRMHelper, '_get_conn')
    @ddt.unpack
    def test_execute(self, mock_get_conn, mock_parse_command,
                     mock_mask_password,
                     check_exit_code=True, exit_code=0):
        mock_parse_command.return_value = (mock.sentinel.parsed_cmd,
                                           mock.sentinel.sanitized_cmd)
        mock_conn = mock_get_conn.return_value
        mock_conn.execute.return_value = (mock.sentinel.stdout,
                                          mock.sentinel.stderr,
                                          exit_code)

        if exit_code == 0 or not check_exit_code:
            result = self._winrm.execute(mock.sentinel.server,
                                         mock.sentinel.command,
                                         check_exit_code=check_exit_code,
                                         retry=False)
            expected_result = (mock.sentinel.stdout, mock.sentinel.stderr)
            self.assertEqual(expected_result, result)
        else:
            self.assertRaises(processutils.ProcessExecutionError,
                              self._winrm.execute,
                              mock.sentinel.server,
                              mock.sentinel.command,
                              check_exit_code=check_exit_code,
                              retry=False)

        mock_get_conn.assert_called_once_with(mock.sentinel.server)
        mock_parse_command.assert_called_once_with(mock.sentinel.command)
        mock_conn.execute.assert_called_once_with(mock.sentinel.parsed_cmd)
        mock_mask_password.assert_has_calls([mock.call(mock.sentinel.stdout),
                                             mock.call(mock.sentinel.stderr)])

    @mock.patch('base64.b64encode')
    @mock.patch.object(strutils, 'mask_password')
    def test_parse_command(self, mock_mask_password, mock_base64):
        mock_mask_password.return_value = mock.sentinel.sanitized_cmd
        mock_base64.return_value = mock.sentinel.encoded_string

        cmd = ('Get-Disk', '-Number', 1)
        result = self._winrm._parse_command(cmd)

        joined_cmd = 'Get-Disk -Number 1'
        expected_command = ("powershell.exe -ExecutionPolicy RemoteSigned "
                            "-NonInteractive -EncodedCommand %s" %
                            mock.sentinel.encoded_string)
        expected_result = expected_command, mock.sentinel.sanitized_cmd

        mock_mask_password.assert_called_once_with(joined_cmd)
        mock_base64.assert_called_once_with(joined_cmd.encode("utf_16_le"))
        self.assertEqual(expected_result, result)

    def _test_get_auth(self, use_cert_auth=False):
        mock_server = {'use_cert_auth': use_cert_auth,
                       'cert_pem_path': mock.sentinel.pem_path,
                       'cert_key_pem_path': mock.sentinel.key_path,
                       'username': mock.sentinel.username,
                       'password': mock.sentinel.password}

        result = self._winrm._get_auth(mock_server)

        expected_result = {'username': mock_server['username']}
        if use_cert_auth:
            expected_result['cert_pem_path'] = mock_server['cert_pem_path']
            expected_result['cert_key_pem_path'] = (
                mock_server['cert_key_pem_path'])
        else:
            expected_result['password'] = mock_server['password']

        self.assertEqual(expected_result, result)

    def test_get_auth_using_certificates(self):
        self._test_get_auth(use_cert_auth=True)

    def test_get_auth_using_password(self):
        self._test_get_auth()


class WinRMConnectionTestCase(test.TestCase):
    @mock.patch.object(winrm_helper, 'setup_winrm')
    @mock.patch.object(winrm_helper, 'winrm')
    @mock.patch.object(winrm_helper.WinRMConnection, '_get_url')
    @mock.patch.object(winrm_helper.WinRMConnection, '_get_default_port')
    def setUp(self, mock_get_port, mock_get_url, mock_winrm,
              mock_setup_winrm):
        self._winrm = winrm_helper.WinRMConnection()
        self._mock_conn = mock_winrm.protocol.Protocol.return_value
        super(WinRMConnectionTestCase, self).setUp()

    @mock.patch.object(winrm_helper, 'setup_winrm')
    @mock.patch.object(winrm_helper, 'winrm')
    @mock.patch.object(winrm_helper.WinRMConnection, '_get_url')
    @mock.patch.object(winrm_helper.WinRMConnection, '_get_default_port')
    def test_init_conn(self, mock_get_port, mock_get_url, mock_winrm,
                       mock_setup_winrm):
        # certificates are passed so we expect cert auth to be used
        cert_auth = True
        winrm_conn = winrm_helper.WinRMConnection(
            ip=mock.sentinel.ip, username=mock.sentinel.username,
            password=mock.sentinel.password,
            cert_pem_path=mock.sentinel.cert_pem_path,
            cert_key_pem_path=mock.sentinel.cert_key_pem_path,
            operation_timeout=mock.sentinel.operation_timeout,
            conn_timeout=mock.sentinel.conn_timeout)

        mock_get_port.assert_called_once_with(cert_auth)
        mock_get_url.assert_called_once_with(mock.sentinel.ip,
                                             mock_get_port.return_value,
                                             cert_auth)
        mock_winrm.protocol.Protocol.assert_called_once_with(
            endpoint=mock_get_url.return_value,
            transport=winrm_helper.TRANSPORT_SSL,
            username=mock.sentinel.username,
            password=mock.sentinel.password,
            cert_pem=mock.sentinel.cert_pem_path,
            cert_key_pem=mock.sentinel.cert_key_pem_path)
        self.assertEqual(mock_winrm.protocol.Protocol.return_value,
                         winrm_conn._conn)
        self.assertEqual(mock.sentinel.conn_timeout,
                         winrm_conn._conn.transport.timeout)
        winrm_conn._conn.set_timeout.assert_called_once_with(
            mock.sentinel.operation_timeout)

    def test_get_default_port_https(self):
        port = self._winrm._get_default_port(use_ssl=True)
        self.assertEqual(winrm_helper.DEFAULT_PORT_HTTPS, port)

    def test_get_default_port_http(self):
        port = self._winrm._get_default_port(use_ssl=False)
        self.assertEqual(winrm_helper.DEFAULT_PORT_HTTP, port)

    def _test_get_url(self, ip=None, use_ssl=True):
        if not ip:
            self.assertRaises(exception.ShareBackendException,
                              self._winrm._get_url,
                              ip=ip,
                              port=mock.sentinel.port,
                              use_ssl=use_ssl)
        else:
            url = self._winrm._get_url(ip=ip,
                                       port=mock.sentinel.port,
                                       use_ssl=use_ssl)
            expected_protocol = 'https' if use_ssl else 'http'
            expected_url = self._winrm._URL_TEMPLATE % dict(
                protocol=expected_protocol,
                port=mock.sentinel.port,
                ip=ip)
            self.assertEqual(expected_url, url)

    def test_get_url_using_ssl(self):
        self._test_get_url(ip=mock.sentinel.ip)

    def test_get_url_using_plaintext(self):
        self._test_get_url(ip=mock.sentinel.ip, use_ssl=False)

    def test_get_url_missing_ip(self):
        self._test_get_url()

    def _test_execute(self, get_output_exception=None):
        self._mock_conn.open_shell.return_value = mock.sentinel.shell_id
        self._mock_conn.run_command.return_value = mock.sentinel.cmd_id

        command_output = (mock.sentinel.stdout,
                          mock.sentinel.stderr,
                          mock.sentinel.exit_code)
        if get_output_exception:
            self._mock_conn.get_command_output.side_effect = (
                get_output_exception)
            self.assertRaises(
                get_output_exception,
                self._winrm.execute,
                mock.sentinel.cmd)
        else:
            self._mock_conn.get_command_output.return_value = command_output
            result = self._winrm.execute(mock.sentinel.cmd)
            self.assertEqual(command_output, result)

        self._mock_conn.open_shell.assert_called_once_with()
        self._mock_conn.run_command.assert_called_once_with(
            mock.sentinel.shell_id, mock.sentinel.cmd)

        self._mock_conn.cleanup_command.assert_called_once_with(
            mock.sentinel.shell_id, mock.sentinel.cmd_id)
        self._mock_conn.close_shell.assert_called_once_with(
            mock.sentinel.shell_id)

    def test_execute(self):
        self._test_execute()

    def test_execute_exception(self):
        self._test_execute(get_output_exception=Exception)



# Copyright 2016 Huawei Inc.
# All Rights Reserved.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock
from oslo_config import cfg
from oslo_serialization import jsonutils
import six
from webob import exc

from manila.api.v2 import share_snapshot_instances
from manila.common import constants
from manila import context
from manila import exception
from manila import policy
from manila import test
from manila.tests.api import fakes
from manila.tests import db_utils
from manila.tests import fake_share

CONF = cfg.CONF


@ddt.ddt
class ShareSnapshotInstancesApiTest(test.TestCase):
    """Share snapshot instance Api Test."""
    def setUp(self):
        super(ShareSnapshotInstancesApiTest, self).setUp()
        self.controller = (share_snapshot_instances.
                           ShareSnapshotInstancesController())
        self.resource_name = self.controller.resource_name
        self.api_version = '2.19'
        self.snapshot_instances_req = fakes.HTTPRequest.blank(
            '/snapshot-instances', version=self.api_version)
        self.admin_context = context.RequestContext('admin', 'fake', True)
        self.member_context = context.RequestContext('fake', 'fake')
        self.snapshot_instances_req.environ['manila.context'] = (
            self.admin_context)
        self.snapshot_instances_req_admin = fakes.HTTPRequest.blank(
            '/snapshot-instances', version=self.api_version,
            use_admin_context=True)
        self.mock_policy_check = self.mock_object(policy, 'check_policy')

    def _get_fake_snapshot_instance(self, summary=False, **values):
        snapshot_instance = fake_share.fake_snapshot_instance(
            as_primitive=True)
        expected_keys = {
            'id',
            'snapshot_id',
            'status',
        }
        expected_snapshot_instance = {key: snapshot_instance[key] for key
                                      in snapshot_instance if key
                                      in expected_keys}

        if not summary:
            expected_snapshot_instance['share_id'] = (
                snapshot_instance.get('share_instance').get('share_id'))
            expected_snapshot_instance.update({
                'created_at': snapshot_instance.get('created_at'),
                'updated_at': snapshot_instance.get('updated_at'),
                'progress': snapshot_instance.get('progress'),
                'provider_location': snapshot_instance.get(
                    'provider_location'),
                'share_instance_id': snapshot_instance.get(
                    'share_instance_id'),
            })

        return snapshot_instance, expected_snapshot_instance

    def _setup_snapshot_instance_data(self, instance=None):
        if instance is None:
            share_instance = db_utils.create_share_instance(
                status=constants.STATUS_AVAILABLE,
                share_id='fake_share_id_1')
            instance = db_utils.create_snapshot_instance(
                'fake_snapshot_id_1',
                status=constants.STATUS_AVAILABLE,
                share_instance_id=share_instance['id'])

        path = '/v2/fake/snapshot-instances/%s/action' % instance['id']
        req = fakes.HTTPRequest.blank(path, version=self.api_version,
                                      script_name=path)
        req.method = 'POST'
        req.headers['content-type'] = 'application/json'
        req.headers['X-Openstack-Manila-Api-Version'] = self.api_version

        return instance, req

    def _get_context(self, role):
        return getattr(self, '%s_context' % role)

    @ddt.data(None, 'FAKE_SNAPSHOT_ID')
    def test_list_snapshot_instances_summary(self, snapshot_id):
        snapshot_instance, expected_snapshot_instance = (
            self._get_fake_snapshot_instance(summary=True))
        self.mock_object(share_snapshot_instances.db,
                         'share_snapshot_instance_get_all_with_filters',
                         mock.Mock(return_value=[snapshot_instance]))

        url = '/snapshot-instances'
        if snapshot_id:
            url += '?snapshot_id=%s' % snapshot_id

        req = fakes.HTTPRequest.blank(url, version=self.api_version)
        req_context = req.environ['manila.context']
        res_dict = self.controller.index(req)

        self.assertEqual([expected_snapshot_instance],
                         res_dict['snapshot_instances'])
        self.mock_policy_check.assert_called_once_with(
            req_context, self.resource_name, 'index')

    def test_list_snapshot_instances_detail(self):
        snapshot_instance, expected_snapshot_instance = (
            self._get_fake_snapshot_instance())
        self.mock_object(share_snapshot_instances.db,
                         'share_snapshot_instance_get_all_with_filters',
                         mock.Mock(return_value=[snapshot_instance]))

        res_dict = self.controller.detail(self.snapshot_instances_req)

        self.assertEqual([expected_snapshot_instance],
                         res_dict['snapshot_instances'])
        self.mock_policy_check.assert_called_once_with(
            self.admin_context, self.resource_name, 'detail')

    def test_list_snapshot_instances_detail_invalid_snapshot(self):
        self.mock_object(share_snapshot_instances.db,
                         'share_snapshot_instance_get_all_with_filters',
                         mock.Mock(return_value=[]))

        req = self.snapshot_instances_req
        req.GET['snapshot_id'] = 'FAKE_SNAPSHOT_ID'

        res_dict = self.controller.detail(req)

        self.assertEqual([], res_dict['snapshot_instances'])
        self.mock_policy_check.assert_called_once_with(
            self.admin_context, self.resource_name, 'detail')

    def test_show(self):
        snapshot_instance, expected_snapshot_instance = (
            self._get_fake_snapshot_instance())
        self.mock_object(
            share_snapshot_instances.db, 'share_snapshot_instance_get',
            mock.Mock(return_value=snapshot_instance))

        res_dict = self.controller.show(self.snapshot_instances_req,
                                        snapshot_instance.get('id'))

        self.assertEqual(expected_snapshot_instance,
                         res_dict['snapshot_instance'])
        self.mock_policy_check.assert_called_once_with(
            self.admin_context, self.resource_name, 'show')

    def test_show_snapshot_instance_not_found(self):
        mock__view_builder_call = self.mock_object(
            share_snapshot_instances.instance_view.ViewBuilder, 'detail')
        fake_exception = exception.ShareSnapshotInstanceNotFound(
            instance_id='FAKE_SNAPSHOT_INSTANCE_ID')
        self.mock_object(share_snapshot_instances.db,
                         'share_snapshot_instance_get',
                         mock.Mock(side_effect=fake_exception))

        self.assertRaises(exc.HTTPNotFound,
                          self.controller.show,
                          self.snapshot_instances_req,
                          'FAKE_SNAPSHOT_INSTANCE_ID')
        self.assertFalse(mock__view_builder_call.called)

    @ddt.data('index', 'detail', 'show', 'reset_status')
    def test_policy_not_authorized(self, method_name):

        method = getattr(self.controller, method_name)
        if method_name in ('index', 'detail'):
            arguments = {}
        else:
            arguments = {
                'id': 'FAKE_SNAPSHOT_ID',
                'body': {'FAKE_KEY': 'FAKE_VAL'},
            }

        noauthexc = exception.PolicyNotAuthorized(action=six.text_type(method))

        with mock.patch.object(
                policy, 'check_policy', mock.Mock(side_effect=noauthexc)):

            self.assertRaises(
                exc.HTTPForbidden, method, self.snapshot_instances_req,
                **arguments)

    @ddt.data('index', 'show', 'detail', 'reset_status')
    def test_upsupported_microversion(self, method_name):
        unsupported_microversions = ('1.0', '2.18')
        method = getattr(self.controller, method_name)
        arguments = {
            'id': 'FAKE_SNAPSHOT_ID',
        }
        if method_name in ('index'):
            arguments.clear()

        for microversion in unsupported_microversions:
            req = fakes.HTTPRequest.blank(
                '/snapshot-instances', version=microversion)
            self.assertRaises(exception.VersionNotFoundForAPIMethod,
                              method, req, **arguments)

    def _reset_status(self, context, instance, req,
                      valid_code=202, valid_status=None, body=None):
        if body is None:
            body = {'reset_status': {'status': constants.STATUS_ERROR}}

        req.body = six.b(jsonutils.dumps(body))
        req.environ['manila.context'] = context

        with mock.patch.object(
                policy, 'check_policy', fakes.mock_fake_admin_check):
            resp = req.get_response(fakes.app())

        # validate response code and model status
        self.assertEqual(valid_code, resp.status_int)

        actual_instance = (
            share_snapshot_instances.db.share_snapshot_instance_get(
                context, instance['id']))
        self.assertEqual(valid_status, actual_instance['status'])

    @ddt.data(*fakes.fixture_reset_status_with_different_roles)
    @ddt.unpack
    def test_reset_status_with_different_roles(self, role, valid_code,
                                               valid_status, version):
        instance, action_req = self._setup_snapshot_instance_data()
        ctxt = self._get_context(role)
        self._reset_status(ctxt, instance, action_req,
                           valid_code=valid_code,
                           valid_status=valid_status)



# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import oslo_messaging as messaging

from manila import rpc
from manila.share import utils


class HuaweiV3API(object):
    """Client side of the huawei V3 rpc API.

    API version history:

        1.0  - Initial version.
    """

    BASE_RPC_API_VERSION = '1.0'

    def __init__(self):
        self.topic = 'huawei_v3'
        target = messaging.Target(topic=self.topic,
                                  version=self.BASE_RPC_API_VERSION)
        self.client = rpc.get_client(target, version_cap='1.0')

    def create_replica_pair(self, context, host, local_share_info,
                            remote_device_wwn, remote_fs_id):
        new_host = utils.extract_host(host)
        call_context = self.client.prepare(server=new_host, version='1.0')
        return call_context.call(
            context, 'create_replica_pair',
            local_share_info=local_share_info,
            remote_device_wwn=remote_device_wwn,
            remote_fs_id=remote_fs_id)



# Copyright 2017 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from manila import test
from manila.wsgi import wsgi


class WSGITestCase(test.TestCase):

    def test_initialize_application(self):
        self.mock_object(wsgi.log, 'register_options')
        self.mock_object(wsgi.cfg.ConfigOpts, '__call__')
        self.mock_object(wsgi.config, 'verify_share_protocols')
        self.mock_object(wsgi.log, 'setup')
        self.mock_object(wsgi.rpc, 'init')
        self.mock_object(wsgi.wsgi, 'Loader')
        wsgi.sys.argv = ['--verbose', '--debug']

        result = wsgi.initialize_application()

        self.assertEqual(
            wsgi.wsgi.Loader.return_value.load_app.return_value, result)
        wsgi.log.register_options.assert_called_once_with(mock.ANY)
        wsgi.cfg.ConfigOpts.__call__.assert_called_once_with(
            mock.ANY, project="manila", version=wsgi.version.version_string())
        wsgi.config.verify_share_protocols.assert_called_once_with()
        wsgi.log.setup.assert_called_once_with(mock.ANY, "manila")
        wsgi.rpc.init.assert_called_once_with(mock.ANY)
        wsgi.wsgi.Loader.assert_called_once_with(mock.ANY)
        wsgi.wsgi.Loader.return_value.load_app.assert_called_once_with(
            name='osapi_share')



# Copyright 2018 Huawei Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""add metadata for access rule

Revision ID: 11ee96se625f3
Revises: 097fad24d2fc
Create Date: 2018-06-16 03:07:15.548947

"""

# revision identifiers, used by Alembic.
revision = '11ee96se625f3'
down_revision = '097fad24d2fc'

from alembic import op
from oslo_log import log
import sqlalchemy as sql

LOG = log.getLogger(__name__)

access_metadata_table_name = 'share_access_rules_metadata'


def upgrade():
    try:
        op.create_table(
            access_metadata_table_name,
            sql.Column('created_at', sql.DateTime(timezone=False)),
            sql.Column('updated_at', sql.DateTime(timezone=False)),
            sql.Column('deleted_at', sql.DateTime(timezone=False)),
            sql.Column('deleted', sql.String(36), default='False'),
            sql.Column('access_id', sql.String(36),
                       sql.ForeignKey('share_access_map.id'), nullable=False),
            sql.Column('key', sql.String(255), nullable=False),
            sql.Column('value', sql.String(1023), nullable=False),
            sql.Column('id', sql.Integer, primary_key=True, nullable=False),
            mysql_engine='InnoDB',
            mysql_charset='utf8'
        )
    except Exception:
        LOG.error("Table |%s| not created!",
                  access_metadata_table_name)
        raise


def downgrade():
    try:
        op.drop_table(access_metadata_table_name)
    except Exception:
        LOG.error("%s table not dropped", access_metadata_table_name)
        raise



# Copyright (c) 2014 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" GlusterFS native protocol (glusterfs) driver for shares.

Manila share is a GlusterFS volume. Unlike the generic driver, this
does not use service VM approach. Instances directly talk with the
GlusterFS backend storage pool. Instance use the 'glusterfs' protocol
to mount the GlusterFS share. Access to the share is allowed via
SSL Certificates. Only the instance which has the SSL trust established
with the GlusterFS backend can mount and hence use the share.

Supports working with multiple glusterfs volumes.
"""

import re

from oslo_log import log

from manila.common import constants
from manila import exception
from manila.i18n import _
from manila.share import driver
from manila.share.drivers.glusterfs import common
from manila.share.drivers.glusterfs import layout
from manila import utils

LOG = log.getLogger(__name__)


ACCESS_TYPE_CERT = 'cert'
AUTH_SSL_ALLOW = 'auth.ssl-allow'
CLIENT_SSL = 'client.ssl'
NFS_EXPORT_VOL = 'nfs.export-volumes'
SERVER_SSL = 'server.ssl'
DYNAMIC_AUTH = 'server.dynamic-auth'


class GlusterfsNativeShareDriver(driver.ExecuteMixin,
                                 layout.GlusterfsShareDriverBase):
    """GlusterFS native protocol (glusterfs) share driver.

    Executes commands relating to Shares.
    Supports working with multiple glusterfs volumes.

    API version history:

        1.0 - Initial version.
        1.1 - Support for working with multiple gluster volumes.
    """

    GLUSTERFS_VERSION_MIN = (3, 6)

    _supported_access_levels = (constants.ACCESS_LEVEL_RW, )
    _supported_access_types = (ACCESS_TYPE_CERT, )
    supported_layouts = ('layout_volume.GlusterfsVolumeMappedLayout',)
    supported_protocols = ('GLUSTERFS',)

    def __init__(self, *args, **kwargs):
        super(GlusterfsNativeShareDriver, self).__init__(
            False, *args, **kwargs)
        self._helpers = None
        self.backend_name = self.configuration.safe_get(
            'share_backend_name') or 'GlusterFS-Native'

    def _setup_via_manager(self, share_mgr, share_mgr_parent=None):
        # Enable gluster volumes for SSL access only.

        gluster_mgr = share_mgr['manager']
        gluster_mgr_parent = (share_mgr_parent or {}).get('manager', None)

        ssl_allow_opt = (gluster_mgr_parent if gluster_mgr_parent else
                         gluster_mgr).get_vol_option(
            AUTH_SSL_ALLOW)
        if not ssl_allow_opt:
            # Not having AUTH_SSL_ALLOW set is a problematic edge case.
            # - In GlusterFS 3.6, it implies that access is allowed to
            #   none, including intra-service access, which causes
            #   problems internally in GlusterFS
            # - In GlusterFS 3.7, it implies that access control is
            #   disabled, which defeats the purpose of this driver --
            # so to avoid these possibilities, we throw an error in this case.
            msg = (_("Option %(option)s is not defined on gluster volume. "
                     "Volume: %(volname)s") %
                   {'volname': gluster_mgr.volume,
                    'option': AUTH_SSL_ALLOW})
            LOG.error(msg)
            raise exception.GlusterfsException(msg)

        gluster_actions = []
        if gluster_mgr_parent:
            # The clone of the snapshot, a new volume, retains the authorized
            # access list of the snapshotted volume/share, which includes TLS
            # identities of the backend servers, Manila hosts and clients.
            # Retain the identities of the GlusterFS servers and Manila host,
            # and exclude those of the clients in the authorized access list of
            # the new volume. The TLS identities of GlusterFS servers are
            # determined as those that are prefixed by 'glusterfs-server'.
            # And the TLS identity of the Manila host is identified as the
            # one that has 'manila-host' as the prefix.
            # Wrt. GlusterFS' parsing of auth.ssl-allow, please see code from
            # https://github.com/gluster/glusterfs/blob/v3.6.2/
            # xlators/protocol/auth/login/src/login.c#L80
            # until end of gf_auth() function
            old_access_list = re.split('[ ,]', ssl_allow_opt)
            glusterfs_server_CN_pattern = '\Aglusterfs-server'
            manila_host_CN_pattern = '\Amanila-host'
            regex = re.compile(
                '%(pattern1)s|%(pattern2)s' % {
                    'pattern1': glusterfs_server_CN_pattern,
                    'pattern2': manila_host_CN_pattern})
            access_to = ','.join(filter(regex.match, old_access_list))
            gluster_actions.append((AUTH_SSL_ALLOW, access_to))

        for option, value in (
            (NFS_EXPORT_VOL, False), (CLIENT_SSL, True), (SERVER_SSL, True)
        ):
            gluster_actions.append((option, value))

        for action in gluster_actions:
            gluster_mgr.set_vol_option(*action)

        gluster_mgr.set_vol_option(DYNAMIC_AUTH, True, ignore_failure=True)

        # SSL enablement requires a fresh volume start
        # to take effect
        if gluster_mgr_parent:
            # in this case the volume is not started
            # yet (will only be started after this func
            # returns), so we have nothing to do here
            pass
        else:
            common._restart_gluster_vol(gluster_mgr)

        return gluster_mgr.export

    @utils.synchronized("glusterfs_native_access", external=False)
    def _update_access_via_manager(self, gluster_mgr, context, share,
                                   add_rules, delete_rules,
                                   recovery=False, share_server=None):
        """Update access rules, authorize SSL CNs (Common Names)."""

        # Fetch existing authorized CNs, the value of Gluster option
        # 'auth.ssl-allow' that is available as a comma separated string.
        # wrt. GlusterFS' parsing of auth.ssl-allow, please see code from
        # https://github.com/gluster/glusterfs/blob/v3.6.2/
        # xlators/protocol/auth/login/src/login.c#L80
        # until end of gf_auth() function
        ssl_allow_opt = gluster_mgr.get_vol_option(AUTH_SSL_ALLOW)

        existing_rules_set = set(re.split('[ ,]', ssl_allow_opt))
        add_rules_set = {rule['access_to'] for rule in add_rules}
        for rule in add_rules_set:
            if re.search('[ ,]', rule):
                raise exception.GlusterfsException(
                    _("Invalid 'access_to' '%s': common names used for "
                      "GlusterFS authentication should not contain comma "
                      "or whitespace.") % rule)
        delete_rules_set = {rule['access_to'] for rule in delete_rules}
        new_rules_set = (
            (existing_rules_set | add_rules_set) - delete_rules_set)

        # Addition or removal of CNs in the authorized list through the
        # Gluster CLI, used by 'GlusterManager' objects, can only be done by
        # replacing the existing list with the newly modified list.
        ssl_allow_opt = ','.join(sorted(new_rules_set))
        gluster_mgr.set_vol_option(AUTH_SSL_ALLOW, ssl_allow_opt)

        # When the Gluster option, DYNAMIC_AUTH is not enabled for the gluster
        # volume/manila share, the removal of CN of a client does not affect
        # the client's existing connection to the volume until the volume is
        # restarted.
        if delete_rules:
            dynauth = gluster_mgr.get_vol_option(DYNAMIC_AUTH, boolean=True)
            if not dynauth:
                common._restart_gluster_vol(gluster_mgr)

    def _update_share_stats(self):
        """Send stats info for the GlusterFS volume."""

        data = dict(
            share_backend_name=self.backend_name,
            vendor_name='Red Hat',
            driver_version='1.1',
            storage_protocol='glusterfs',
            reserved_percentage=self.configuration.reserved_share_percentage)

        # We don't use a service mount to get stats data.
        # Instead we use glusterfs quota feature and use that to limit
        # the share to its expected share['size'].

        # TODO(deepakcs): Change below once glusterfs supports volume
        # specific stats via the gluster cli.
        data['total_capacity_gb'] = 'unknown'
        data['free_capacity_gb'] = 'unknown'

        super(GlusterfsNativeShareDriver, self)._update_share_stats(data)

    def get_network_allocations_number(self):
        return 0



# Copyright 2013 OpenStack Foundation
# Copyright 2015 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ipaddress
import six
import socket

from oslo_config import cfg
from oslo_log import log

from manila.common import constants
from manila import exception
from manila.i18n import _
from manila import network
from manila.network.neutron import api as neutron_api
from manila.network.neutron import constants as neutron_constants
from manila import utils

LOG = log.getLogger(__name__)

neutron_network_plugin_opts = [
    cfg.StrOpt(
        'neutron_physical_net_name',
        help="The name of the physical network to determine which net segment "
             "is used. This opt is optional and will only be used for "
             "networks configured with multiple segments.",
        deprecated_group='DEFAULT'),
]

neutron_single_network_plugin_opts = [
    cfg.StrOpt(
        'neutron_net_id',
        help="Default Neutron network that will be used for share server "
             "creation. This opt is used only with "
             "class 'NeutronSingleNetworkPlugin'.",
        deprecated_group='DEFAULT'),
    cfg.StrOpt(
        'neutron_subnet_id',
        help="Default Neutron subnet that will be used for share server "
             "creation. Should be assigned to network defined in opt "
             "'neutron_net_id'. This opt is used only with "
             "class 'NeutronSingleNetworkPlugin'.",
        deprecated_group='DEFAULT'),
]

neutron_bind_network_plugin_opts = [
    cfg.StrOpt(
        'neutron_vnic_type',
        help="vNIC type used for binding.",
        choices=['baremetal', 'normal', 'direct',
                 'direct-physical', 'macvtap'],
        default='baremetal'),
    cfg.StrOpt(
        "neutron_host_id",
        help="Host ID to be used when creating neutron port. If not set "
             "host is set to manila-share host by default.",
        default=socket.gethostname()),
]

neutron_binding_profile = [
    cfg.ListOpt(
        "neutron_binding_profiles",
        help="A list of binding profiles to be used during port binding. This "
        "option can be used with the NeutronBindNetworkPlugin. The value for "
        "this option has to be a comma separated list of names that "
        "correspond to each binding profile. Each binding profile needs to be "
        "specified as an individual configuration section using the binding "
        "profile name as the section name."),
]

neutron_binding_profile_opts = [
    cfg.StrOpt(
        'neutron_switch_id',
        help="Switch ID for binding profile."),
    cfg.StrOpt(
        'neutron_port_id',
        help="Port ID on the given switch.",),
    cfg.DictOpt(
        'neutron_switch_info',
        help="Switch label. For example: 'switch_ip: 10.4.30.5'. Multiple "
             "key-value pairs separated by commas are accepted.",),
]

CONF = cfg.CONF


class NeutronNetworkPlugin(network.NetworkBaseAPI):

    def __init__(self, *args, **kwargs):
        db_driver = kwargs.pop('db_driver', None)
        config_group_name = kwargs.get('config_group_name', 'DEFAULT')
        super(NeutronNetworkPlugin,
              self).__init__(config_group_name=config_group_name,
                             db_driver=db_driver)
        self._neutron_api = None
        self._neutron_api_args = args
        self._neutron_api_kwargs = kwargs
        self._label = kwargs.pop('label', 'user')
        CONF.register_opts(
            neutron_network_plugin_opts,
            group=self.neutron_api.config_group_name)

    @property
    def label(self):
        return self._label

    @property
    @utils.synchronized("instantiate_neutron_api")
    def neutron_api(self):
        if not self._neutron_api:
            self._neutron_api = neutron_api.API(*self._neutron_api_args,
                                                **self._neutron_api_kwargs)
        return self._neutron_api

    def _store_neutron_net_info(self, context, share_network):
        self._save_neutron_network_data(context, share_network)
        self._save_neutron_subnet_data(context, share_network)

    def allocate_network(self, context, share_server, share_network=None,
                         **kwargs):
        """Allocate network resources using given network information.

        Create neutron ports for a given neutron network and subnet,
        create manila db records for allocated neutron ports.

        :param context: RequestContext object
        :param share_network: share network data
        :param kwargs: allocations parameters given by the back-end
                       driver. Supported params:
                       'count' - how many allocations should be created
                       'device_owner' - set owner for network allocations
        :rtype: list of :class: 'dict'
        """
        if not self._has_provider_network_extension():
            msg = "%s extension required" % neutron_constants.PROVIDER_NW_EXT
            raise exception.NetworkBadConfigurationException(reason=msg)

        self._verify_share_network(share_server['id'], share_network)
        self._store_neutron_net_info(context, share_network)

        allocation_count = kwargs.get('count', 1)
        device_owner = kwargs.get('device_owner', 'share')

        ports = []
        for __ in range(0, allocation_count):
            ports.append(self._create_port(context, share_server,
                                           share_network, device_owner))

        return ports

    def manage_network_allocations(self, context, allocations, share_server,
                                   share_network=None):

        self._verify_share_network(share_server['id'], share_network)
        self._store_neutron_net_info(context, share_network)

        # We begin matching the allocations to known neutron ports and
        # finally return the non-consumed allocations
        remaining_allocations = list(allocations)

        fixed_ip_filter = 'subnet_id=' + share_network['neutron_subnet_id']

        port_list = self.neutron_api.list_ports(
            network_id=share_network['neutron_net_id'],
            device_owner='manila:share',
            fixed_ips=fixed_ip_filter)

        selected_ports = self._get_ports_respective_to_ips(
            remaining_allocations, port_list)

        LOG.debug("Found matching allocations in Neutron:"
                  " %s", six.text_type(selected_ports))

        for selected_port in selected_ports:
            port_dict = {
                'id': selected_port['port']['id'],
                'share_server_id': share_server['id'],
                'ip_address': selected_port['allocation'],
                'gateway': share_network['gateway'],
                'mac_address': selected_port['port']['mac_address'],
                'status': constants.STATUS_ACTIVE,
                'label': self.label,
                'network_type': share_network.get('network_type'),
                'segmentation_id': share_network.get('segmentation_id'),
                'ip_version': share_network['ip_version'],
                'cidr': share_network['cidr'],
                'mtu': share_network['mtu'],
            }

            # There should not be existing allocations with the same port_id.
            try:
                existing_port = self.db.network_allocation_get(
                    context, selected_port['port']['id'], read_deleted=False)
            except exception.NotFound:
                pass
            else:
                msg = _("There were existing conflicting manila network "
                        "allocations found while trying to manage share "
                        "server %(new_ss)s. The conflicting port belongs to "
                        "share server %(old_ss)s.") % {
                            'new_ss': share_server['id'],
                            'old_ss': existing_port['share_server_id'],
                    }
                raise exception.ManageShareServerError(reason=msg)

            # If there are previously deleted allocations, we undelete them
            try:
                self.db.network_allocation_get(
                    context, selected_port['port']['id'], read_deleted=True)
            except exception.NotFound:
                self.db.network_allocation_create(context, port_dict)
            else:
                port_dict.pop('id')
                port_dict.update({
                    'deleted_at': None,
                    'deleted': 'False',
                })
                self.db.network_allocation_update(
                    context, selected_port['port']['id'], port_dict,
                    read_deleted=True)

            remaining_allocations.remove(selected_port['allocation'])

        return remaining_allocations

    def unmanage_network_allocations(self, context, share_server_id):

        ports = self.db.network_allocations_get_for_share_server(
            context, share_server_id)

        for port in ports:
            self.db.network_allocation_delete(context, port['id'])

    def _get_ports_respective_to_ips(self, allocations, port_list):

        selected_ports = []

        for port in port_list:
            for ip in port['fixed_ips']:
                if ip['ip_address'] in allocations:
                    if not any(port['id'] == p['port']['id']
                               for p in selected_ports):
                        selected_ports.append(
                            {'port': port, 'allocation': ip['ip_address']})
                    else:
                        LOG.warning("Port %s has more than one IP that "
                                    "matches allocations, please use ports "
                                    "respective to only one allocation IP.",
                                    port['id'])

        return selected_ports

    def _get_matched_ip_address(self, fixed_ips, ip_version):
        """Get first ip address which matches the specified ip_version."""

        for ip in fixed_ips:
            try:
                address = ipaddress.ip_address(six.text_type(ip['ip_address']))
                if address.version == ip_version:
                    return ip['ip_address']
            except ValueError:
                LOG.error("%(address)s isn't a valid ip "
                          "address, omitted."), {'address':
                                                 ip['ip_address']}
        msg = _("Can not find any IP address with configured IP "
                "version %(version)s in share-network.") % {'version':
                                                            ip_version}
        raise exception.NetworkBadConfigurationException(reason=msg)

    def deallocate_network(self, context, share_server_id):
        """Deallocate neutron network resources for the given share server.

        Delete previously allocated neutron ports, delete manila db
        records for deleted ports.

        :param context: RequestContext object
        :param share_server_id: id of share server
        :rtype: None
        """
        ports = self.db.network_allocations_get_for_share_server(
            context, share_server_id)

        for port in ports:
            self._delete_port(context, port)

    def _get_port_create_args(self, share_server, share_network,
                              device_owner):
        return {
            "network_id": share_network['neutron_net_id'],
            "subnet_id": share_network['neutron_subnet_id'],
            "device_owner": 'manila:' + device_owner,
            "device_id": share_server.get('id'),
        }

    def _create_port(self, context, share_server, share_network, device_owner):
        create_args = self._get_port_create_args(share_server, share_network,
                                                 device_owner)

        port = self.neutron_api.create_port(
            share_network['project_id'], **create_args)

        ip_address = self._get_matched_ip_address(port['fixed_ips'],
                                                  share_network['ip_version'])
        port_dict = {
            'id': port['id'],
            'share_server_id': share_server['id'],
            'ip_address': ip_address,
            'gateway': share_network['gateway'],
            'mac_address': port['mac_address'],
            'status': constants.STATUS_ACTIVE,
            'label': self.label,
            'network_type': share_network.get('network_type'),
            'segmentation_id': share_network.get('segmentation_id'),
            'ip_version': share_network['ip_version'],
            'cidr': share_network['cidr'],
            'mtu': share_network['mtu'],
        }
        return self.db.network_allocation_create(context, port_dict)

    def _delete_port(self, context, port):
        try:
            self.neutron_api.delete_port(port['id'])
        except exception.NetworkException:
            self.db.network_allocation_update(
                context, port['id'], {'status': constants.STATUS_ERROR})
            raise
        else:
            self.db.network_allocation_delete(context, port['id'])

    def _has_provider_network_extension(self):
        extensions = self.neutron_api.list_extensions()
        return neutron_constants.PROVIDER_NW_EXT in extensions

    def _is_neutron_multi_segment(self, share_network, net_info=None):
        if net_info is None:
            net_info = self.neutron_api.get_network(
                share_network['neutron_net_id'])
        return 'segments' in net_info

    def _save_neutron_network_data(self, context, share_network):
        net_info = self.neutron_api.get_network(
            share_network['neutron_net_id'])
        segmentation_id = None
        network_type = None

        if self._is_neutron_multi_segment(share_network, net_info):
            # we have a multi segment network and need to identify the
            # lowest segment used for binding
            phy_nets = []
            phy = self.neutron_api.configuration.neutron_physical_net_name
            if not phy:
                msg = "Cannot identify segment used for binding. Please add "
                "neutron_physical_net_name in configuration."
                raise exception.NetworkBadConfigurationException(reason=msg)
            for segment in net_info['segments']:
                phy_nets.append(segment['provider:physical_network'])
                if segment['provider:physical_network'] == phy:
                    segmentation_id = segment['provider:segmentation_id']
                    network_type = segment['provider:network_type']
            if not (segmentation_id and network_type):
                msg = ("No matching neutron_physical_net_name found for %s "
                       "(found: %s)." % (phy, phy_nets))
                raise exception.NetworkBadConfigurationException(reason=msg)
        else:
            network_type = net_info['provider:network_type']
            segmentation_id = net_info['provider:segmentation_id']

        provider_nw_dict = {
            'network_type': network_type,
            'segmentation_id': segmentation_id,
            'mtu':  net_info['mtu'],
        }
        share_network.update(provider_nw_dict)

        if self.label != 'admin':
            self.db.share_network_update(
                context, share_network['id'], provider_nw_dict)

    def _save_neutron_subnet_data(self, context, share_network):
        subnet_info = self.neutron_api.get_subnet(
            share_network['neutron_subnet_id'])

        subnet_values = {
            'cidr': subnet_info['cidr'],
            'gateway': subnet_info['gateway_ip'],
            'ip_version': subnet_info['ip_version']
        }
        share_network.update(subnet_values)

        if self.label != 'admin':
            self.db.share_network_update(
                context, share_network['id'], subnet_values)


class NeutronSingleNetworkPlugin(NeutronNetworkPlugin):

    def __init__(self, *args, **kwargs):
        super(NeutronSingleNetworkPlugin, self).__init__(*args, **kwargs)
        CONF.register_opts(
            neutron_single_network_plugin_opts,
            group=self.neutron_api.config_group_name)
        self.net = self.neutron_api.configuration.neutron_net_id
        self.subnet = self.neutron_api.configuration.neutron_subnet_id
        self._verify_net_and_subnet()

    def _select_proper_share_network(self, context, share_network):
        if self.label != 'admin':
            share_network = self._update_share_network_net_data(
                context, share_network)
        else:
            share_network = {
                'project_id': self.neutron_api.admin_project_id,
                'neutron_net_id': self.net,
                'neutron_subnet_id': self.subnet,
            }
        return share_network

    def allocate_network(self, context, share_server, share_network=None,
                         **kwargs):

        share_network = self._select_proper_share_network(
            context, share_network)

        return super(NeutronSingleNetworkPlugin, self).allocate_network(
            context, share_server, share_network, **kwargs)

    def manage_network_allocations(self, context, allocations, share_server,
                                   share_network=None):

        share_network = self._select_proper_share_network(
            context, share_network)

        return super(NeutronSingleNetworkPlugin,
                     self).manage_network_allocations(
            context, allocations, share_server, share_network)

    def _verify_net_and_subnet(self):
        data = dict(net=self.net, subnet=self.subnet)
        if self.net and self.subnet:
            net = self.neutron_api.get_network(self.net)
            if not (net.get('subnets') and data['subnet'] in net['subnets']):
                raise exception.NetworkBadConfigurationException(
                    "Subnet '%(subnet)s' does not belong to "
                    "network '%(net)s'." % data)
        else:
            raise exception.NetworkBadConfigurationException(
                "Neutron net and subnet are expected to be both set. "
                "Got: net=%(net)s and subnet=%(subnet)s." % data)

    def _update_share_network_net_data(self, context, share_network):
        upd = dict()

        if not share_network.get('neutron_net_id') == self.net:
            if share_network.get('neutron_net_id') is not None:
                raise exception.NetworkBadConfigurationException(
                    "Using neutron net id different from None or value "
                    "specified in the config is forbidden for "
                    "NeutronSingleNetworkPlugin. Allowed values: (%(net)s, "
                    "None), received value: %(err)s" % {
                        "net": self.net,
                        "err": share_network.get('neutron_net_id')})
            upd['neutron_net_id'] = self.net
        if not share_network.get('neutron_subnet_id') == self.subnet:
            if share_network.get('neutron_subnet_id') is not None:
                raise exception.NetworkBadConfigurationException(
                    "Using neutron subnet id different from None or value "
                    "specified in the config is forbidden for "
                    "NeutronSingleNetworkPlugin. Allowed values: (%(snet)s, "
                    "None), received value: %(err)s" % {
                        "snet": self.subnet,
                        "err": share_network.get('neutron_subnet_id')})
            upd['neutron_subnet_id'] = self.subnet
        if upd:
            share_network = self.db.share_network_update(
                context, share_network['id'], upd)
        return share_network


class NeutronBindNetworkPlugin(NeutronNetworkPlugin):
    def __init__(self, *args, **kwargs):
        super(NeutronBindNetworkPlugin, self).__init__(*args, **kwargs)

        self.binding_profiles = []
        CONF.register_opts(
            neutron_binding_profile,
            group=self.neutron_api.config_group_name)
        conf = CONF[self.neutron_api.config_group_name]
        if conf.neutron_binding_profiles:
            for profile in conf.neutron_binding_profiles:
                CONF.register_opts(neutron_binding_profile_opts, group=profile)
                self.binding_profiles.append(profile)

        CONF.register_opts(
            neutron_bind_network_plugin_opts,
            group=self.neutron_api.config_group_name)
        self.config = self.neutron_api.configuration

    def update_network_allocation(self, context, share_server):
        if self.config.neutron_vnic_type == 'normal':
            ports = self.db.network_allocations_get_for_share_server(
                context,
                share_server['id'])
            self._wait_for_ports_bind(ports, share_server)
            return ports

    @utils.retry(exception.NetworkBindException, retries=20)
    def _wait_for_ports_bind(self, ports, share_server):
        inactive_ports = []
        for port in ports:
            port = self._neutron_api.show_port(port['id'])
            if (port['status'] == neutron_constants.PORT_STATUS_ERROR or
                    ('binding:vif_type' in port and
                     port['binding:vif_type'] ==
                     neutron_constants.VIF_TYPE_BINDING_FAILED)):
                msg = _("Port binding %s failed.") % port['id']
                raise exception.NetworkException(msg)
            elif port['status'] != neutron_constants.PORT_STATUS_ACTIVE:
                LOG.debug("The port %(id)s is in state %(state)s. "
                          "Wait for active state.", {
                              "id": port['id'],
                              "state": port['status']})
                inactive_ports.append(port['id'])
        if len(inactive_ports) == 0:
            return
        msg = _("Ports are not fully bound for share server "
                "'%(s_id)s' (inactive ports: %(ports)s)") % {
            "s_id": share_server['id'],
            "ports": inactive_ports}
        raise exception.NetworkBindException(msg)

    def _get_port_create_args(self, share_server, share_network,
                              device_owner):
        arguments = super(
            NeutronBindNetworkPlugin, self)._get_port_create_args(
                share_network, share_network, device_owner)
        arguments['host_id'] = self.config.neutron_host_id
        arguments['binding:vnic_type'] = self.config.neutron_vnic_type
        if self.binding_profiles:
            local_links = []
            for profile in self.binding_profiles:
                local_links.append({
                    'switch_id': CONF[profile]['neutron_switch_id'],
                    'port_id': CONF[profile]['neutron_port_id'],
                    'switch_info': CONF[profile]['neutron_switch_info'],
                })

            arguments['binding:profile'] = {
                "local_link_information": local_links}
        return arguments

    def _save_neutron_network_data(self, context, share_network):
        """Store the Neutron network info.

        In case of dynamic multi segments the segment is determined while
        binding the port. Therefore this method will return for multi segments
        network without storing network information (apart from mtu).

        Instead, multi segments network will wait until ports are bound and
        then store network information (see allocate_network()).
        """
        if self._is_neutron_multi_segment(share_network):
            # In case of dynamic multi segment the segment is determined while
            # binding the port, only mtu is known and already needed
            self._save_neutron_network_mtu(context, share_network)
            return
        super(NeutronBindNetworkPlugin, self)._save_neutron_network_data(
            context, share_network)

    def _save_neutron_network_mtu(self, context, share_network):
        """Store the Neutron network mtu.

        In case of dynamic multi segments only the mtu needs storing before
        binding the port.
        """
        net_info = self.neutron_api.get_network(
            share_network['neutron_net_id'])

        mtu_dict = {
            'mtu':  net_info['mtu'],
        }
        share_network.update(mtu_dict)

        if self.label != 'admin':
            self.db.share_network_update(
                context, share_network['id'], mtu_dict)

    def allocate_network(self, context, share_server, share_network=None,
                         **kwargs):
        ports = super(NeutronBindNetworkPlugin, self).allocate_network(
            context, share_server, share_network, **kwargs)
        # If vnic type is 'normal' we expect a neutron agent to bind the
        # ports. This action requires a vnic to be spawned by the driver.
        # Therefore we do not wait for the port binding here, but
        # return the unbound ports and expect the share manager to call
        # update_network_allocation after the share server was created, in
        # order to update the ports with the correct binding.
        if self.config.neutron_vnic_type != 'normal':
            self._wait_for_ports_bind(ports, share_server)
            if self._is_neutron_multi_segment(share_network):
                # update segment information after port bind
                super(NeutronBindNetworkPlugin,
                      self)._save_neutron_network_data(context, share_network)
                for num, port in enumerate(ports):
                    port_info = {
                        'network_type': share_network['network_type'],
                        'segmentation_id': share_network['segmentation_id'],
                        'cidr': share_network['cidr'],
                        'ip_version': share_network['ip_version'],
                    }
                    ports[num] = self.db.network_allocation_update(
                        context, port['id'], port_info)
        return ports


class NeutronBindSingleNetworkPlugin(NeutronSingleNetworkPlugin,
                                     NeutronBindNetworkPlugin):
    pass



# Copyright 2016 Alex Meade
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import datetime

import ddt
import mock
from oslo_config import cfg
from oslo_serialization import jsonutils
from oslo_utils import uuidutils
import six
import webob

from manila.api.openstack import wsgi
from manila.api.v2 import share_group_snapshots
from manila.common import constants
from manila import context
from manila import db
from manila import exception
from manila import policy
from manila import test
from manila.tests.api import fakes
from manila.tests import db_utils

CONF = cfg.CONF


@ddt.ddt
class ShareGroupSnapshotAPITest(test.TestCase):

    def setUp(self):
        super(ShareGroupSnapshotAPITest, self).setUp()
        self.controller = share_group_snapshots.ShareGroupSnapshotController()
        self.resource_name = self.controller.resource_name
        self.api_version = '2.31'
        self.mock_policy_check = self.mock_object(
            policy, 'check_policy', mock.Mock(return_value=True))
        self.request = fakes.HTTPRequest.blank(
            '/share-groups', version=self.api_version, experimental=True)
        self.context = self.request.environ['manila.context']
        self.admin_context = context.RequestContext('admin', 'fake', True)
        self.member_context = context.RequestContext('fake', 'fake')
        self.flags(transport_url='rabbit://fake:fake@mqhost:5672')

    def _get_fake_share_group_snapshot(self, **values):
        snap = {
            'id': 'fake_id',
            'user_id': 'fakeuser',
            'project_id': 'fakeproject',
            'status': constants.STATUS_CREATING,
            'name': None,
            'description': None,
            'share_group_id': None,
            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
            'members': [],
        }

        snap.update(**values)

        expected_snap = copy.deepcopy(snap)
        del expected_snap['user_id']
        return snap, expected_snap

    def _get_fake_simple_share_group_snapshot(self, **values):
        snap = {
            'id': 'fake_id',
            'name': None,
        }

        snap.update(**values)
        expected_snap = copy.deepcopy(snap)
        return snap, expected_snap

    def _get_fake_share_group_snapshot_member(self, **values):
        member = {
            'id': 'fake_id',
            'user_id': 'fakeuser',
            'project_id': 'fakeproject',
            'status': constants.STATUS_CREATING,
            'share_group_snapshot_id': None,
            'share_proto': None,
            'share_id': None,
            'size': None,
            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
        }

        member.update(**values)

        expected_member = copy.deepcopy(member)
        del expected_member['user_id']
        del expected_member['status']
        expected_member['share_protocol'] = member['share_proto']
        del expected_member['share_proto']
        return member, expected_member

    def test_create_invalid_body(self):
        body = {"not_group_snapshot": {}}

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.create, self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_no_share_group_id(self):
        body = {"share_group_snapshot": {}}

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.create, self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        fake_id = six.text_type(uuidutils.generate_uuid())
        body = {"share_group_snapshot": {"share_group_id": fake_id}}
        mock_create = self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(return_value=fake_snap))

        res_dict = self.controller.create(self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')
        mock_create.assert_called_once_with(
            self.context, share_group_id=fake_id)
        res_dict['share_group_snapshot'].pop('links')

        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])

    def test_create_group_does_not_exist(self):
        fake_id = six.text_type(uuidutils.generate_uuid())
        body = {"share_group_snapshot": {"share_group_id": fake_id}}
        self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(side_effect=exception.ShareGroupNotFound(
                share_group_id=six.text_type(uuidutils.generate_uuid()))))

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.create, self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_group_does_not_a_uuid(self):
        self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(side_effect=exception.ShareGroupNotFound(
                share_group_id='not_a_uuid',
            )))
        body = {"share_group_snapshot": {"share_group_id": "not_a_uuid"}}

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.create, self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_invalid_share_group(self):
        fake_id = six.text_type(uuidutils.generate_uuid())
        body = {"share_group_snapshot": {"share_group_id": fake_id}}
        self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(side_effect=exception.InvalidShareGroup(
                reason='bad_status')))

        self.assertRaises(
            webob.exc.HTTPConflict, self.controller.create, self.request, body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_with_name(self):
        fake_name = 'fake_name'
        fake_snap, expected_snap = self._get_fake_share_group_snapshot(
            name=fake_name)
        fake_id = six.text_type(uuidutils.generate_uuid())
        mock_create = self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(return_value=fake_snap))
        body = {
            "share_group_snapshot": {
                "share_group_id": fake_id,
                "name": fake_name,
            }
        }
        res_dict = self.controller.create(self.request, body)

        res_dict['share_group_snapshot'].pop('links')

        mock_create.assert_called_once_with(
            self.context, share_group_id=fake_id, name=fake_name)
        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_with_description(self):
        fake_description = 'fake_description'
        fake_snap, expected_snap = self._get_fake_share_group_snapshot(
            description=fake_description)
        fake_id = six.text_type(uuidutils.generate_uuid())
        mock_create = self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(return_value=fake_snap))

        body = {
            "share_group_snapshot": {
                "share_group_id": fake_id,
                "description": fake_description,
            }
        }
        res_dict = self.controller.create(self.request, body)

        res_dict['share_group_snapshot'].pop('links')

        mock_create.assert_called_once_with(
            self.context, share_group_id=fake_id, description=fake_description)
        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_create_with_name_and_description(self):
        fake_name = 'fake_name'
        fake_description = 'fake_description'
        fake_id = six.text_type(uuidutils.generate_uuid())
        fake_snap, expected_snap = self._get_fake_share_group_snapshot(
            description=fake_description, name=fake_name)
        mock_create = self.mock_object(
            self.controller.share_group_api, 'create_share_group_snapshot',
            mock.Mock(return_value=fake_snap))

        body = {
            "share_group_snapshot": {
                "share_group_id": fake_id,
                "description": fake_description,
                "name": fake_name,
            }
        }
        res_dict = self.controller.create(self.request, body)

        res_dict['share_group_snapshot'].pop('links')

        mock_create.assert_called_once_with(
            self.context, share_group_id=fake_id, name=fake_name,
            description=fake_description)
        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'create')

    def test_update_with_name_and_description(self):
        fake_name = 'fake_name'
        fake_description = 'fake_description'
        fake_id = six.text_type(uuidutils.generate_uuid())
        fake_snap, expected_snap = self._get_fake_share_group_snapshot(
            description=fake_description, name=fake_name)
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(return_value=fake_snap))
        mock_update = self.mock_object(
            self.controller.share_group_api, 'update_share_group_snapshot',
            mock.Mock(return_value=fake_snap))

        body = {
            "share_group_snapshot": {
                "description": fake_description,
                "name": fake_name,
            }
        }
        res_dict = self.controller.update(self.request, fake_id, body)

        res_dict['share_group_snapshot'].pop('links')

        mock_update.assert_called_once_with(
            self.context, fake_snap,
            {"name": fake_name, "description": fake_description})
        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'update')

    def test_update_snapshot_not_found(self):
        body = {"share_group_snapshot": {}}
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(side_effect=exception.NotFound))

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.update, self.request, 'fake_id', body)

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'update')

    def test_update_invalid_body(self):
        body = {"not_group_snapshot": {}}
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.update,
                          self.request, 'fake_id', body)
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'update')

    def test_update_invalid_body_invalid_field(self):
        body = {"share_group_snapshot": {"unknown_field": ""}}
        exc = self.assertRaises(webob.exc.HTTPBadRequest,
                                self.controller.update,
                                self.request, 'fake_id', body)
        self.assertIn('unknown_field', six.text_type(exc))
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'update')

    def test_update_invalid_body_readonly_field(self):
        body = {"share_group_snapshot": {"created_at": []}}
        exc = self.assertRaises(webob.exc.HTTPBadRequest,
                                self.controller.update,
                                self.request, 'fake_id', body)
        self.assertIn('created_at', six.text_type(exc))
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'update')

    def test_list_index(self):
        fake_snap, expected_snap = self._get_fake_simple_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap]))

        res_dict = self.controller.index(self.request)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual([expected_snap], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get_all')

    def test_list_index_no_share_groups(self):
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[]))

        res_dict = self.controller.index(self.request)

        self.assertEqual([], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get_all')

    def test_list_index_with_limit(self):
        fake_snap, expected_snap = self._get_fake_simple_share_group_snapshot()
        fake_snap2, expected_snap2 = (
            self._get_fake_simple_share_group_snapshot(
                id="fake_id2"))
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap, fake_snap2]))
        req = fakes.HTTPRequest.blank('/share-group-snapshots?limit=1',
                                      version=self.api_version,
                                      experimental=True)
        req_context = req.environ['manila.context']

        res_dict = self.controller.index(req)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual(1, len(res_dict['share_group_snapshots']))
        self.assertEqual([expected_snap], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            req_context, self.resource_name, 'get_all')

    def test_list_index_with_limit_and_offset(self):
        fake_snap, expected_snap = self._get_fake_simple_share_group_snapshot()
        fake_snap2, expected_snap2 = (
            self._get_fake_simple_share_group_snapshot(id="fake_id2"))
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap, fake_snap2]))
        req = fakes.HTTPRequest.blank(
            '/share-group-snapshots?limit=1&offset=1',
            version=self.api_version, experimental=True)
        req_context = req.environ['manila.context']

        res_dict = self.controller.index(req)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual(1, len(res_dict['share_group_snapshots']))
        self.assertEqual([expected_snap2], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            req_context, self.resource_name, 'get_all')

    def test_list_detail(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap]))

        res_dict = self.controller.detail(self.request)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual(1, len(res_dict['share_group_snapshots']))
        self.assertEqual(expected_snap, res_dict['share_group_snapshots'][0])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get_all')

    def test_list_detail_no_share_groups(self):
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[]))
        res_dict = self.controller.detail(self.request)
        self.assertEqual([], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get_all')

    def test_list_detail_with_limit(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        fake_snap2, expected_snap2 = self._get_fake_share_group_snapshot(
            id="fake_id2")
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap, fake_snap2]))
        req = fakes.HTTPRequest.blank('/share-group-snapshots?limit=1',
                                      version=self.api_version,
                                      experimental=True)
        req_context = req.environ['manila.context']

        res_dict = self.controller.detail(req)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual(1, len(res_dict['share_group_snapshots']))
        self.assertEqual([expected_snap], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            req_context, self.resource_name, 'get_all')

    def test_list_detail_with_limit_and_offset(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        fake_snap2, expected_snap2 = self._get_fake_share_group_snapshot(
            id="fake_id2")
        self.mock_object(
            self.controller.share_group_api, 'get_all_share_group_snapshots',
            mock.Mock(return_value=[fake_snap, fake_snap2]))
        req = fakes.HTTPRequest.blank(
            '/share-group-snapshots?limit=1&offset=1',
            version=self.api_version,
            experimental=True)
        req_context = req.environ['manila.context']

        res_dict = self.controller.detail(req)

        res_dict['share_group_snapshots'][0].pop('links')

        self.assertEqual(1, len(res_dict['share_group_snapshots']))
        self.assertEqual([expected_snap2], res_dict['share_group_snapshots'])
        self.mock_policy_check.assert_called_once_with(
            req_context, self.resource_name, 'get_all')

    def test_delete(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(return_value=fake_snap))
        self.mock_object(
            self.controller.share_group_api, 'delete_share_group_snapshot')

        res = self.controller.delete(self.request, fake_snap['id'])

        self.assertEqual(202, res.status_code)
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'delete')

    def test_delete_not_found(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(side_effect=exception.NotFound))

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.delete, self.request, fake_snap['id'])

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'delete')

    def test_delete_in_conflicting_status(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(return_value=fake_snap))
        self.mock_object(
            self.controller.share_group_api, 'delete_share_group_snapshot',
            mock.Mock(side_effect=exception.InvalidShareGroupSnapshot(
                reason='blah')))

        self.assertRaises(
            webob.exc.HTTPConflict,
            self.controller.delete, self.request, fake_snap['id'])

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'delete')

    def test_show(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(return_value=fake_snap))

        res_dict = self.controller.show(self.request, fake_snap['id'])

        res_dict['share_group_snapshot'].pop('links')

        self.assertEqual(expected_snap, res_dict['share_group_snapshot'])
        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get')

    def test_show_share_group_not_found(self):
        fake_snap, expected_snap = self._get_fake_share_group_snapshot()
        self.mock_object(
            self.controller.share_group_api, 'get_share_group_snapshot',
            mock.Mock(side_effect=exception.NotFound))

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.show, self.request, fake_snap['id'])

        self.mock_policy_check.assert_called_once_with(
            self.context, self.resource_name, 'get')

    def _get_context(self, role):
        return getattr(self, '%s_context' % role)

    def _setup_share_group_snapshot_data(self, share_group_snapshot=None,
                                         version='2.31'):
        if share_group_snapshot is None:
            share_group_snapshot = db_utils.create_share_group_snapshot(
                'fake_id', status=constants.STATUS_AVAILABLE)

        path = ('/v2/fake/share-group-snapshots/%s/action' %
                share_group_snapshot['id'])
        req = fakes.HTTPRequest.blank(path, script_name=path, version=version)
        req.headers[wsgi.API_VERSION_REQUEST_HEADER] = version
        req.headers[wsgi.EXPERIMENTAL_API_REQUEST_HEADER] = 'True'
        return share_group_snapshot, req

    @ddt.data(*fakes.fixture_force_delete_with_different_roles)
    @ddt.unpack
    def test_share_group_snapshot_force_delete_with_different_roles(
            self, role,  resp_code, version):
        group_snap, req = self._setup_share_group_snapshot_data()
        ctxt = self._get_context(role)
        req.method = 'POST'
        req.headers['content-type'] = 'application/json'
        action_name = 'force_delete'
        body = {action_name: {'status': constants.STATUS_ERROR}}
        req.body = six.b(jsonutils.dumps(body))
        req.headers['X-Openstack-Manila-Api-Version'] = self.api_version
        req.environ['manila.context'] = ctxt

        with mock.patch.object(
                policy, 'check_policy', fakes.mock_fake_admin_check):
            resp = req.get_response(fakes.app())

        # Validate response
        self.assertEqual(resp_code, resp.status_int)

    @ddt.data(*fakes.fixture_reset_status_with_different_roles)
    @ddt.unpack
    def test_share_group_snapshot_reset_status_with_different_roles(
            self, role, valid_code, valid_status, version):
        ctxt = self._get_context(role)
        group_snap, req = self._setup_share_group_snapshot_data()
        action_name = 'reset_status'
        body = {action_name: {'status': constants.STATUS_ERROR}}
        req.method = 'POST'
        req.headers['content-type'] = 'application/json'
        req.body = six.b(jsonutils.dumps(body))
        req.headers['X-Openstack-Manila-Api-Version'] = self.api_version
        req.environ['manila.context'] = ctxt

        with mock.patch.object(
                policy, 'check_policy', fakes.mock_fake_admin_check):
            resp = req.get_response(fakes.app())

        # Validate response code and model status
        self.assertEqual(valid_code, resp.status_int)

        actual_model = db.share_group_snapshot_get(ctxt, group_snap['id'])
        self.assertEqual(valid_status, actual_model['status'])



# Copyright 2019 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock
import webob

from manila.api.v2 import share_servers
from manila.common import constants
from manila.db import api as db_api
from manila import exception
from manila import policy
from manila.share import api as share_api
from manila import test
from manila.tests.api import fakes
from manila.tests import db_utils
from manila import utils


@ddt.ddt
class ShareServerControllerTest(test.TestCase):
    """Share server api test"""

    def setUp(self):
        super(ShareServerControllerTest, self).setUp()
        self.mock_policy_check = self.mock_object(
            policy, 'check_policy', mock.Mock(return_value=True))
        self.controller = share_servers.ShareServerController()
        self.resource_name = self.controller.resource_name

    @ddt.data(constants.STATUS_ACTIVE, constants.STATUS_ERROR,
              constants.STATUS_DELETING, constants.STATUS_CREATING,
              constants.STATUS_MANAGING, constants.STATUS_UNMANAGING,
              constants.STATUS_UNMANAGE_ERROR, constants.STATUS_MANAGE_ERROR)
    def test_share_server_reset_status(self, status):
        req = fakes.HTTPRequest.blank('/v2/share-servers/fake-share-server/',
                                      use_admin_context=True)
        body = {'reset_status': {'status': status}}

        context = req.environ['manila.context']
        mock_update = self.mock_object(db_api, 'share_server_update')

        result = self.controller.share_server_reset_status(
            req, 'fake_server_id', body)

        self.assertEqual(202, result.status_int)
        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'reset_status')
        mock_update.assert_called_once_with(
            context, 'fake_server_id', {'status': status})

    def test_share_reset_server_status_invalid(self):
        req = fakes.HTTPRequest.blank('/reset_status', use_admin_context=True)
        body = {'reset_status': {'status': constants.STATUS_EXTENDING}}
        context = req.environ['manila.context']

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.share_server_reset_status,
            req, id='fake_server_id', body=body)
        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'reset_status')

    def test_share_server_reset_status_no_body(self):
        req = fakes.HTTPRequest.blank('/reset_status', use_admin_context=True)
        context = req.environ['manila.context']

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.share_server_reset_status,
            req, id='fake_server_id', body={})
        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'reset_status')

    def test_share_server_reset_status_no_status(self):
        req = fakes.HTTPRequest.blank('/reset_status', use_admin_context=True)
        context = req.environ['manila.context']

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller.share_server_reset_status,
            req, id='fake_server_id', body={'reset_status': {}})
        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'reset_status')

    def _setup_manage_test_request_body(self):
        body = {
            'share_network_id': 'fake_net_id',
            'host': 'fake_host',
            'identifier': 'fake_identifier',
            'driver_options': {'opt1': 'fake_opt1', 'opt2': 'fake_opt2'},
        }
        return body

    @ddt.data('fake_net_name', '')
    def test_manage(self, share_net_name):
        """Tests share server manage"""
        req = fakes.HTTPRequest.blank('/v2/share-servers/',
                                      use_admin_context=True,
                                      version="2.49")
        context = req.environ['manila.context']
        share_network = db_utils.create_share_network(name=share_net_name)
        share_server = db_utils.create_share_server(
            share_network_id=share_network['id'],
            host='fake_host',
            identifier='fake_identifier',
            is_auto_deletable=False)

        self.mock_object(db_api, 'share_network_get', mock.Mock(
            return_value=share_network))
        self.mock_object(utils, 'validate_service_host')

        body = {
            'share_server': self._setup_manage_test_request_body()
        }

        manage_share_server_mock = self.mock_object(
            share_api.API, 'manage_share_server',
            mock.Mock(return_value=share_server))

        result = self.controller.manage(req, body)
        expected_result = {
            'share_server': {
                'id': share_server['id'],
                'project_id': 'fake',
                'updated_at': None,
                'status': constants.STATUS_ACTIVE,
                'host': 'fake_host',
                'share_network_id': share_server['share_network_id'],
                'created_at': share_server['created_at'],
                'backend_details': {},
                'identifier': share_server['identifier'],
                'is_auto_deletable': share_server['is_auto_deletable'],
            }
        }
        if share_net_name != '':
            expected_result['share_server']['share_network_name'] = (
                'fake_net_name')
        else:
            expected_result['share_server']['share_network_name'] = (
                share_server['share_network_id'])

        req_params = body['share_server']
        manage_share_server_mock.assert_called_once_with(
            context, req_params['identifier'], req_params['host'],
            share_network, req_params['driver_options'])

        self.assertEqual(expected_result, result)

        self.mock_policy_check.assert_called_once_with(
            context, self.resource_name, 'manage_share_server')

    def test_manage_invalid(self):
        req = fakes.HTTPRequest.blank('/manage_share_server',
                                      use_admin_context=True)
        context = req.environ['manila.context']
        share_network = db_utils.create_share_network()

        body = {
            'share_server': self._setup_manage_test_request_body()
        }
        self.mock_object(utils, 'validate_service_host')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))

        manage_share_server_mock = self.mock_object(
            share_api.API, 'manage_share_server',
            mock.Mock(side_effect=exception.InvalidInput('foobar')))

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.manage, req, body)

        req_params = body['share_server']
        manage_share_server_mock.assert_called_once_with(
            context, req_params['identifier'], req_params['host'],
            share_network, req_params['driver_options'])

    def test_manage_forbidden(self):
        """Tests share server manage without admin privileges"""
        req = fakes.HTTPRequest.blank('/manage_share_server')
        self.mock_object(share_api.API, 'manage_share_server', mock.Mock())
        error = mock.Mock(side_effect=exception.PolicyNotAuthorized(action=''))
        self.mock_object(share_api.API, 'manage_share_server', error)

        body = {
            'share_server': self._setup_manage_test_request_body()
        }

        self.assertRaises(webob.exc.HTTPForbidden,
                          self.controller.manage,
                          req,
                          body)

    def test__validate_manage_share_server_validate_no_body(self):
        """Tests share server manage"""
        req = fakes.HTTPRequest.blank('/manage')
        body = {}

        self.assertRaises(webob.exc.HTTPUnprocessableEntity,
                          self.controller.manage,
                          req,
                          body)

    @ddt.data({'empty': False, 'key': 'host'},
              {'empty': False, 'key': 'share_network_id'},
              {'empty': False, 'key': 'identifier'},
              {'empty': True, 'key': 'host'},
              {'empty': True, 'key': 'share_network_id'},
              {'empty': True, 'key': 'identifier'})
    @ddt.unpack
    def test__validate_manage_share_server_validate_without_parameters(
            self, empty, key):
        """Tests share server manage without some parameters"""
        req = fakes.HTTPRequest.blank('/manage_share_server')
        self.mock_object(share_api.API, 'manage_share_server', mock.Mock())

        body = {
            'share_server': self._setup_manage_test_request_body(),
        }

        if empty:
            body['share_server'][key] = None
        else:
            body['share_server'].pop(key)

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.manage,
                          req,
                          body)

    @ddt.data(
        (webob.exc.HTTPBadRequest, exception.ServiceNotFound('foobar')),
        (webob.exc.HTTPBadRequest, exception.ServiceIsDown('foobar')),
        (webob.exc.HTTPForbidden, exception.PolicyNotAuthorized('foobar')),
        (webob.exc.HTTPForbidden, exception.AdminRequired())
    )
    @ddt.unpack
    def test__validate_manage_share_server_validate_service_host(
            self, exception_to_raise, side_effect_exception):
        req = fakes.HTTPRequest.blank('/manage')
        context = req.environ['manila.context']
        error = mock.Mock(side_effect=side_effect_exception)
        self.mock_object(utils, 'validate_service_host', error)

        self.assertRaises(
            exception_to_raise, self.controller.manage, req,
            {'share_server': self._setup_manage_test_request_body()})

        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'manage_share_server')

    def test__validate_manage_share_server_share_network_not_found(self):
        req = fakes.HTTPRequest.blank('/manage')
        context = req.environ['manila.context']
        self.mock_object(utils, 'validate_service_host')
        error = mock.Mock(
            side_effect=exception.ShareNetworkNotFound(share_network_id="foo"))
        self.mock_object(db_api, 'share_network_get', error)
        body = self._setup_manage_test_request_body()

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.manage,
                          req,
                          {'share_server': body})

        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'manage_share_server')

    def test__validate_manage_share_server_driver_opts_not_instance_dict(self):
        req = fakes.HTTPRequest.blank('/manage')
        context = req.environ['manila.context']
        self.mock_object(utils, 'validate_service_host')
        self.mock_object(db_api, 'share_network_get')
        body = self._setup_manage_test_request_body()
        body['driver_options'] = 'incorrect'
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.manage,
                          req,
                          {'share_server': body})

        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'manage_share_server')

    def test__validate_manage_share_server_error_extract_host(self):
        req = fakes.HTTPRequest.blank('/manage')
        context = req.environ['manila.context']
        body = self._setup_manage_test_request_body()
        body['host'] = 'fake@backend#pool'
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.manage,
                          req,
                          {'share_server': body})

        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'manage_share_server')

    @ddt.data(True, False)
    def test_unmanage(self, force):
        server = self._setup_unmanage_tests()
        req = fakes.HTTPRequest.blank('/unmanage')
        context = req.environ['manila.context']
        mock_get = self.mock_object(
            db_api, 'share_server_get', mock.Mock(return_value=server))
        mock_unmanage = self.mock_object(
            share_api.API, 'unmanage_share_server',
            mock.Mock(return_value=202))
        body = {'unmanage': {'force': force}}
        resp = self.controller.unmanage(req, server['id'], body)

        self.assertEqual(202, resp.status_int)

        mock_get.assert_called_once_with(context, server['id'])
        mock_unmanage.assert_called_once_with(context, server, force=force)

    def test_unmanage_share_server_not_found(self):
        """Tests unmanaging share servers"""
        req = fakes.HTTPRequest.blank('/v2/share-servers/fake_server_id/')
        context = req.environ['manila.context']
        share_server_error = mock.Mock(
            side_effect=exception.ShareServerNotFound(
                share_server_id='fake_server_id'))
        get_mock = self.mock_object(
            db_api, 'share_server_get', share_server_error)
        body = {'unmanage': {'force': True}}

        self.assertRaises(webob.exc.HTTPNotFound,
                          self.controller.unmanage,
                          req,
                          'fake_server_id',
                          body)

        get_mock.assert_called_once_with(context, 'fake_server_id')

    @ddt.data(constants.STATUS_MANAGING, constants.STATUS_DELETING,
              constants.STATUS_CREATING, constants.STATUS_UNMANAGING)
    def test_unmanage_share_server_invalid_statuses(self, status):
        """Tests unmanaging share servers"""
        server = self._setup_unmanage_tests(status=status)
        get_mock = self.mock_object(db_api, 'share_server_get',
                                    mock.Mock(return_value=server))
        req = fakes.HTTPRequest.blank('/unmanage_share_server')
        context = req.environ['manila.context']
        body = {'unmanage': {'force': True}}

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.unmanage,
                          req,
                          server['id'],
                          body)

        get_mock.assert_called_once_with(context, server['id'])

    def _setup_unmanage_tests(self, status=constants.STATUS_ACTIVE):
        server = db_utils.create_share_server(
            id='fake_server_id', status=status)
        self.mock_object(db_api, 'share_server_get',
                         mock.Mock(return_value=server))
        return server

    @ddt.data(exception.ShareServerInUse, exception.PolicyNotAuthorized)
    def test_unmanage_share_server_badrequest(self, exc):
        req = fakes.HTTPRequest.blank('/unmanage')
        server = self._setup_unmanage_tests()
        context = req.environ['manila.context']
        error = mock.Mock(side_effect=exc('foobar'))
        mock_unmanage = self.mock_object(
            share_api.API, 'unmanage_share_server', error)
        body = {'unmanage': {'force': True}}

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.unmanage,
                          req,
                          'fake_server_id',
                          body)

        mock_unmanage.assert_called_once_with(context, server, force=True)
        policy.check_policy.assert_called_once_with(
            context, self.resource_name, 'unmanage_share_server')



# Copyright 2013 OpenStack Foundation
# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import abc

from oslo_config import cfg
from oslo_utils import importutils
import six

from manila.db import base as db_base
from manila import exception
from manila.i18n import _

network_opts = [
    cfg.StrOpt(
        'network_api_class',
        default='manila.network.neutron.'
                'neutron_network_plugin.NeutronNetworkPlugin',
        deprecated_group='DEFAULT',
        help='The full class name of the Networking API class to use.'),
]

network_base_opts = [
    cfg.BoolOpt(
        'network_plugin_ipv4_enabled',
        default=True,
        help="Whether to support IPv4 network resource, Default=True."),
    cfg.BoolOpt(
        'network_plugin_ipv6_enabled',
        default=False,
        help="Whether to support IPv6 network resource, Default=False. "
             "If this option is True, the value of "
             "'network_plugin_ipv4_enabled' will be ignored."),
]

CONF = cfg.CONF


def API(config_group_name=None, label='user'):
    """Selects class and config group of network plugin.

    :param config_group_name: name of config group to be used for
                              registration of networking opts.
    :returns: instance of networking plugin class
    """
    CONF.register_opts(network_opts, group=config_group_name)
    if config_group_name:
        network_api_class = getattr(CONF, config_group_name).network_api_class
    else:
        network_api_class = CONF.network_api_class
    cls = importutils.import_class(network_api_class)
    return cls(config_group_name=config_group_name, label=label)


@six.add_metaclass(abc.ABCMeta)
class NetworkBaseAPI(db_base.Base):
    """User network plugin for setting up main net interfaces."""

    def __init__(self, config_group_name=None, db_driver=None):
        if config_group_name:
            CONF.register_opts(network_base_opts,
                               group=config_group_name)
        else:
            CONF.register_opts(network_base_opts)
        self.configuration = getattr(CONF,
                                     six.text_type(config_group_name), CONF)
        super(NetworkBaseAPI, self).__init__(db_driver=db_driver)

    def _verify_share_network(self, share_server_id, share_network):
        if share_network is None:
            msg = _("'Share network' is not provided for setting up "
                    "network interfaces for 'Share server' "
                    "'%s'.") % share_server_id
            raise exception.NetworkBadConfigurationException(reason=msg)

    def update_network_allocation(self, context, share_server):
        """Update network allocation.

        Optional method to be called by the manager after share server creation
        which can be overloaded in case the port state has to be updated.

        :param context: RequestContext object
        :param share_server: share server object
        :return: list of updated ports or None if nothing was updated
        """

    @abc.abstractmethod
    def allocate_network(self, context, share_server, share_network=None,
                         **kwargs):
        pass

    @abc.abstractmethod
    def deallocate_network(self, context, share_server_id):
        pass

    @abc.abstractmethod
    def manage_network_allocations(self, context, allocations, share_server,
                                   share_network=None):
        pass

    @abc.abstractmethod
    def unmanage_network_allocations(self, context, share_server_id):
        pass

    @property
    def enabled_ip_versions(self):
        if not hasattr(self, '_enabled_ip_versions'):
            self._enabled_ip_versions = set()
            if self.configuration.network_plugin_ipv6_enabled:
                self._enabled_ip_versions.add(6)
            if self.configuration.network_plugin_ipv4_enabled:
                self._enabled_ip_versions.add(4)
            if not self._enabled_ip_versions:
                msg = _("Either 'network_plugin_ipv4_enabled' or "
                        "'network_plugin_ipv6_enabled' "
                        "should be configured to 'True'.")
                raise exception.NetworkBadConfigurationException(reason=msg)
        return self._enabled_ip_versions



# Copyright 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from manila.network.linux import ovs_lib
from manila import test


class OVS_Lib_Test(test.TestCase):
    """A test suite to exercise the OVS libraries."""

    def setUp(self):
        super(OVS_Lib_Test, self).setUp()
        self.BR_NAME = "br-int"
        self.TO = "--timeout=2"

        self.br = ovs_lib.OVSBridge(self.BR_NAME)
        self.execute_p = mock.patch('manila.utils.execute')
        self.execute = self.execute_p.start()

    def tearDown(self):
        self.execute_p.stop()
        super(OVS_Lib_Test, self).tearDown()

    def test_reset_bridge(self):
        self.br.reset_bridge()
        self.execute.assert_has_calls([mock.call("ovs-vsctl", self.TO, "--",
                                                 "--if-exists", "del-br",
                                                 self.BR_NAME,
                                                 run_as_root=True),
                                       mock.call("ovs-vsctl", self.TO,
                                                 "add-br",
                                                 self.BR_NAME,
                                                 run_as_root=True)])

    def test_delete_port(self):
        pname = "tap5"
        self.br.delete_port(pname)
        self.execute.assert_called_once_with("ovs-vsctl", self.TO, "--",
                                             "--if-exists", "del-port",
                                             self.BR_NAME, pname,
                                             run_as_root=True)

    def test_port_id_regex(self):
        result = ('external_ids        : {attached-mac="fa:16:3e:23:5b:f2",'
                  ' iface-id="5c1321a7-c73f-4a77-95e6-9f86402e5c8f",'
                  ' iface-status=active}\nname                :'
                  ' "dhc5c1321a7-c7"\nofport              : 2\n')
        match = self.br.re_id.search(result)
        vif_mac = match.group('vif_mac')
        vif_id = match.group('vif_id')
        port_name = match.group('port_name')
        ofport = int(match.group('ofport'))
        self.assertEqual('fa:16:3e:23:5b:f2', vif_mac)
        self.assertEqual('5c1321a7-c73f-4a77-95e6-9f86402e5c8f', vif_id)
        self.assertEqual('dhc5c1321a7-c7', port_name)
        self.assertEqual(2, ofport)
